"administrator",
"airline pilot",
"artist",
"attorney",
"bricklayer",
"bus driver",
"carpenter",
"chemist",
"clerk",
"clown",
"college student",
"consultant",
"cook",
"corporate liquidator",
"curator",
"dentist",
"designer",
"director",
"doctor",
"driver",
"electrician",
"engineer",
"farmer",
"fireman",
"gardener",
"geophysicist",
"guard",
"high school student",
"hs student",
"human resource consultant",
"investment banker",
"janitor",
"jr college student",
"juggler and unicyclist",
"junior college student",
"lawyer",
"letter carrier",
"librarian",
"mafia killer",
"mathematician",
"mechanic",
"midwife",
"milkman",
"nurse",
"orthodontist",
"pacifist",
"painter",
"petroleum engineer",
"physician",
"physicist",
"police officer",
"politician",
"prison guard",
"professor",
"programmer",
"researcher",
"sailor",
"sanitation worker",
"sculptor",
"secretary",
"security guard",
"semiotician",
"soldier",
"sql programmer",
"stock broker",
"student",
"systems analyst",
"teacher",
"teller",
"vulcanologist",
"writer",
