"+%=x/,? 73 just testing you.",
",?/=+9% 73 just testing you.",
",?/=q+% 73 just testing u.",
"8y67fvgb oops cat on keyboard.",
"9p0ojgbhnw23 oops cat on keyboard.",
"%w=+/,? 73 just testing you.",
"A clumsy gecko just fell on my hand.",
"Are you a communist?",
"Are you a democrat?",
"Are you a libertarian?",
"Are you a republican?",
"Are you divorced?",
"Are you good at auto repair?",
"Are you good at calculus? I have a question for you.",
"Are you good at car repair? I have a question for you.",
"Are you good at group theory? I have a question for you.",
"Are you good at summing infinite series?",
"Are you married?",
"Are you on daylight savings time?",
"Are you single?",
"Are you using a linear amplifier?",
"Are you using an iambic key?",
"Are you widowed?",
"Aye, laddie, have you ever been to sea?",
"Can you pick kryptonite locks? Those are tricky.",
"Can you pick kryptonite locks?",
"Can you pick master locks? Those are supposed to be easy.",
"Can you think of something else interesting to say?",
"Copy?",
"Cpy?",
"Did you attend ORU?",
"Did you hear news?",
"Did you hear the news?",
"Do u understand English?",
"Do you have a baby?",
"Do you have a cat?",
"Do you have a dog?",
"Do you have a ferret?",
"Do you have a gecko?",
"Do you have a horse?",
"Do you know morse code?",
"Do you know the word copacetic?",
"Do you know the word garrulous?",
"Do you know the word prosy?",
"Do you know what a pangram is?",
"Do you like bach?",
"Do you like bernstein?",
"Do you like c and w music?",
"Do you like classical music?",
"Do you like country music?",
"Do you like hawaiian music?",
"Do you like jawaiian music?",
"Do you like modern rock?",
"Do you like mozart?",
"Do you like pop music?",
"Do you like reggae music?",
"Do you like rock music?",
"Do you like swing music?",
"Do you listen to Deutsche Welle?",
"Do you listen to Radio Australia?",
"Do you listen to Radio Moscow?",
"Do you listen to Radio New Zealand Intl?",
"Do you listen to Radio Peace Peoples Republic of Korea?",
"Do you listen to VOA?",
"Do you listen to click and clack on PBS?",
"Do you listen to the BBC?",
"Do you prefer metric units?",
"Do you recycle often?",
"Do you understand English?",
"Do you watch Quantum Leap?",
"Do you watch STTNG?",
"Do you watch Simpsons?",
"Gaverete Ruskie?",
"Habla espanol?",
"Has the war started yet?",
"Have a nice T day.",
"Have a nice thanksgiving.",
"Have you been to Hawaii recently?",
"Have you been to Kauai since Iniki?",
"Have you ever been to New Madrid, Missouri?",
"Have you ever been to the Chatham islands?",
"Have you ever contacted Kalawao county, Molokai?",
"Have you ever contacted Loving county, Texas?",
"Have you ever played lemmings?",
"Have you ever played tetris?",
"Have you ever seen a computer virus?",
"How copy so far?",
"How copy?",
"How did you learn morse code?",
"How is my signal?",
"How is the weather?",
"How long have you been a radio amateur?",
"How old is your rig?",
"Hows my signal?",
"Hows the weather?",
"I am mobile and am going to the office.",
"I am mobile and am going to work.",
"I am mobile and driving to work.",
"I am mobile, driving cross country.",
"I am mobile, driving home from work.",
"I am mobile, riding a Peruvian Paso Fino horse.",
"I am mobile, riding a Peruvian Paso horse.",
"I am mobile, stuck in traffic.",
"I am using a linear amplifier.",
"I am using an iambic key.",
"I just saw a bolide.",
"I live at 10 Downing.",
"I live at 1010 Springwood.",
"I live at 1620 Keeaumoku.",
"I live at 2500 Pennsylvania.",
"I live at 61C Escondido.",
"I sometimes monitor ULF for earthquake precursors.",
"I sometimes monitor ulf for earthquake precursors.",
"I sometimes work packet on vhf.",
"Is it green there or brown?",
"Is the bad news true?",
"Is your state the one that doesnt have any city governments?",
"Is your state the one that has a unicameral legislature?",
"Is your state the one that has boroughs instead of counties?",
"Is your state the one that has parishes instead of counties?",
"Is your state the one that has townships instead of counties?",
"Ive fallen and I cant get up.",
"Jocks find quartz glyph, vex BMW.",
"Jocks find quartz glyph, vex BMW.",
"Jocks find quartz glyph, vex BMW.",
"Lots of QRM.",
"Mele kelikimaka and hauoli makahiki hou 1995.",
"Must QRT for Babylon V.",
"Must QRT for Star Trek.",
"Must QRT for bathroom break.",
"Must QRT for breakfast.",
"Must QRT for cold breakfast.",
"Must QRT for cold dinner.",
"Must QRT for cold supper.",
"Must QRT for diarreah break.",
"Must QRT for dinner.",
"Must QRT for hot breakfast.",
"Must QRT for hot dinner.",
"Must QRT for hot lunch.",
"Must QRT for hot supper.",
"Must QRT for lunch.",
"Must QRT for sleep.",
"Must QRT for some sleep.",
"Must QRT for supper.",
"Must QRT invading tanks making too much noise outside to hear you.",
"Must QRT occupation forces knocking at door.",
"Must QRT to hear news about approaching hurricane.",
"Must QRT wind has blown door open again.",
"Must QRT, lightning storm has started.",
"Must QRT, lightning threatens.",
"Must QRT, nuclear attack?",
"Must QRT, tornado sirens sounding.",
"Must QRT, tsunami sirens sounding.",
"Must QRT, weather radio warbling.",
"Our neighbors have a horse named morse code.",
"Our neighbors have a horse named morse.",
"Please QRS.",
"Propagation is good.",
"Propagation is poor.",
"Propagation is very good.",
"Propagation is very poor.",
"QLF?",
"QRG",
"QRH",
"QRI 1",
"QRI 2",
"QRI 3",
"QRI",
"QRJ",
"QRK 1",
"QRK 2",
"QRK 3",
"QRK 4",
"QRK 5",
"QRK",
"QRL",
"QRM",
"QRM?",
"QRN 1",
"QRN 2",
"QRN 3",
"QRN 4",
"QRN 5",
"QRN",
"QRO",
"QRP",
"QRP?",
"QRQ 10",
"QRQ 15",
"QRQ 20",
"QRQ 5",
"QRQ",
"QRS 10",
"QRS 15",
"QRS 20",
"QRS 5",
"QRS",
"QRS?",
"QRU",
"QRV",
"QRW",
"QRX 0300",
"QRX 0500",
"QRX 2000",
"QRX 2300",
"QRX",
"QRX?",
"QRY 1",
"QRY 2",
"QRY 3",
"QRY",
"QRZ",
"QSA 1",
"QSA 2",
"QSA 3",
"QSA 4",
"QSA 5",
"QSA",
"QSB",
"QSD",
"QSG 1",
"QSG 2",
"QSG",
"QSK",
"QSK?",
"QSL",
"QSM",
"QSN",
"QSO",
"QSP",
"QST",
"QSU",
"QSW",
"QSX",
"QSY",
"QSY?",
"QSZ",
"QTB",
"QTC",
"QTH?",
"QTR",
"QTR?",
"Should we QSY?",
"So how is the weather?",
"Sorry, stack of books just fell over.",
"Sprechen Sie Deutsch?",
"Thanks for the call.",
"There is a rainbow outside the window.",
"There is an eclipse of the moon right now.",
"Tnx for the call.",
"Tnx for the report.",
"Tnx for ur call.",
"We are on vacation and I am mobile.",
"We just felt a small quake here.",
"What are your hobbies?",
"What is the air pressure there?",
"What is the air quality there?",
"What is the air temperature there?",
"What is the humidity there?",
"What is your QTH?",
"What is your elevation?",
"What is your hobby?",
"What is your job?",
"What is your sign? I am a cancer.",
"What is your sign? I am a centaur.",
"What is your sign? I am a fish.",
"What time zone are you in?",
"Whats your job?",
"When did you start to learn morse code?",
"Where do you live now?",
"Where do you live?",
"Where is Hollister, California anyway?",
"Where is Parkfield, California anyway?",
"Whoa, we are having an earthquake here. A real roller.",
"Would you have given Morse a Nobel prize?",
"You are my first contact today.",
"You are my last contact today.",
"You are my second contact today.",
"You are using a nonlinear amplifier?",
"You are using an iambic key?",
"Your contest sequence no is 1035.",
"Your contest sequence no is 12.",
"Your contest sequence no is 17.",
"Your contest sequence no is 479.",
"Your contest sequence no is 73.",
"Your contest sequence no is 7v.",
"Your contest sequence no is 826.",
"Your contest sequence no is 99.",
"Your job is?",
"Your signal is chirpy.",
"am using a linear amplifier.",
"ewsqgbhjolpi oops cat on keyboard.",
"ru using linear amplifier?",
