"10 meter dipole",
"10 meter whip",
"3 element beam",
"5 band vertical",
"5 element loop",
"7 band beam",
"beam",
"delta loop",
"dipole",
"doublet zepp",
"dummy load",
"ground=plane",
"half wave dipole",
"inverted V",
"light bulb",
"log periodic",
"long=wire",
"monobander",
"parasitic beam",
"quad loop",
"quad vertical",
"quagi",
"quarter wave vertical",
"random wire up 2 kite",
"random wire up to kite",
"random wire",
"rhombic",
"symmetrical delta loop",
"trap doublet",
"tribander",
"whip",
"yagi",
"zepp",
