"a blue norther",
"a noreaster",
"a siberia express",
"chinook winds",
"cloudy",
"cold and windy",
"cold/dry",
"cool/windy",
"damp",
"drizzling",
"fog/drizzle",
"foggy",
"gale force winds",
"gentle rain",
"gusting trade winds",
"hailing",
"horizontal rain",
"hot and humid",
"hot and windy",
"hot",
"hot/dry",
"hot/humid",
"hot/muggy",
"hot/smoggy",
"humid",
"kona winds",
"pleasantly cool",
"pleasantly warm",
"raining cats and dogs",
"raining",
"santa anna winds",
"sleeting",
"smoggy",
"snowing",
"spitting rain",
"sunny",
"tornadic winds",
"trade winds",
"uncomfortably hot",
"very cold",
"very hot",
"very windy",
"warm",
"wet",
"wet/foggy",
"wet/windy",
"windy and warm",
"windy",
