"Collins 75S=3",
"Collins KWM=2",
"Drake T=4XB",
"Drake TR3",
"Drake TR4",
"Eico 753",
"Hallicrafters SR150",
"Hallicrafters SR160",
"Hallicrafters SR2000",
"Hallicrafters SR400",
"Hallicrafters SR500",
"Heathkit DX=660B",
"Heathkit HG=10B",
"Heathkit HR=10",
"Heathkit HW100",
"Heathkit HW12",
"Heathkit HW12A",
"Heathkit HW22",
"Heathkit HW22A",
"Heathkit HW32",
"Heathkit HW32A",
"Heathkit HW8",
"Heathkit SB100",
"Homebrew",
"Icom IC=1275",
"Icom IC=275",
"Icom IC=475",
"Icom IC=575",
"Icom IC=725",
"Icom IC=726",
"Icom IC=730",
"Icom IC=735",
"Icom IC=751",
"Icom IC=751A",
"Icom IC=765",
"Icom IC=781",
"Kenwood TR=450",
"Kenwood TR=751",
"Kenwood TR=850",
"Kenwood TR=851",
"Kenwood TS=140",
"Kenwood TS=440",
"Kenwood TS=680",
"Kenwood TS=711",
"Kenwood TS=77",
"Kenwood TS=850",
"National NCL 200",
"National NCX3",
"National NCX5",
"RS HTX=100",
"Radio Shack HTX=100",
"Swan 120",
"Swan 140",
"Swan 180",
"Swan 240",
"Swan 350",
"Ten Tec 562",
"Ten Tec 585",
"Ten Tec Argonaut",
"Ten Tec Delta",
"Ten Tec Omni",
"Yaesu FT=1000",
"Yaesu FT=101",
"Yaesu FT=736",
"Yaesu FT=747",
"Yaesu FT=757",
"Yaesu FT=767",
"Yaesu FT=840",
"Yaesu FT=990",
