"AA0ET",
"AA4LR",
"AA5BT",
"AA6NP",
"AA6YD",
"AA9BK",
"AB5AP",
"AB6FI",
"AC4HF",
"G0GWA",
"GM4ZNX",
"I5FLN",
"IK5AAX",
"JA1BLV",
"JY1",
"K2WK",
"K5RC",
"K6OCK",
"K6XO",
"K9ALD",
"KA0WCH",
"KA1AXY",
"KA1CV",
"KA1UTU",
"KA1ZGC",
"KA2RAF",
"KA6S",
"KB0CY",
"KB2NRH",
"KB6CSP",
"KB6JXT",
"KC6SKV",
"KC6SSS",
"KC6TDR",
"KC6VWV",
"KD3FU",
"KD4AUS",
"KD4CPL",
"KE2TP",
"KE4ZV",
"KF6BZF",
"KF8NH",
"KJ6NN",
"KJ9U",
"KJ9U/KH6",
"KK6JQ",
"KM3T",
"KT7H",
"N1AL",
"N1JCX",
"N4PBK",
"N4VRN",
"N5IAL",
"N5OP",
"N6BIS",
"N6MWC",
"N6TTO",
"N8EMR",
"N9FZX",
"N9JRV",
"N9LFF",
"N9MYI",
"NE3X",
"NH6EG",
"NH6EG/5",
"NH6ZW",
"NJ7E",
"NR3Z",
"NT1G",
"OD5NG",
"OH3BK",
"OH8NUP",
"TG9VT",
"UT5RP",
"VE6MGS",
"VE7EMD",
"VK2BQS",
"VK2EG",
"W0PBV",
"W0RIJ",
"W0RSB",
"W2FG",
"W2JGR",
"W3GRG",
"W3OTC",
"WA1UXA",
"WA2AGE",
"WA3EEC",
"WA3TBL",
"WA3UQV",
"WA5RPF",
"WA6DGX",
"WA7VYJ",
"WA8ZGO",
"WB2CJL",
"WB4JCM",
"WB5FDP",
"WB5FKH",
"WB5NRN",
"WB7CJO",
"WB7EEL",
"WB7TZA",
"WB8EOH",
"WB9IVR",
"WD0EIB",
"WH6UR",
"WH6UR/5",
"WX9T",
